/*****************************************************************************
 * OBSLUHA VIACERYCH KLIENTOV SUCASNE (DCERSKE PROCESY)
 * Internetova (AF_INET) komunikacia Client-Server (toto je server)
 * Je tu spravene vetvenie procesu - fork() :
 *
 * 1. deklaracia socketu; socket()
 * 2. spoj socket s datovou strukturou; bind()
 * 3. nastavenie akceptovania pripojeni a max. pocet sucasne akceptovatelnych
 *    pripojeni; accept()
 * 4. cakanie na spojenie; listen()
 *****************************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <signal.h>
#include <unistd.h>

#define PORT 9734		/* port, na kt. sa clienti pripajaju */
#define BACKLOG 5		/* max. pocet sucasnych pripojeni */
#define MAXLEN 255

int
main ()
{
  int sockfd_s, sockfd_c;
  int client_len, len, addr_len;
  struct sockaddr_in addr_s, addr_c;
  char buffer[MAXLEN];

  if ((sockfd_s = socket (AF_INET, SOCK_STREAM, 0)) == -1)	/* TCP */
    {
      perror ("Chyba pri vytvoreni socketu");
      exit (1);
    }
  
  addr_s.sin_family = AF_INET;
  addr_s.sin_addr.s_addr = INADDR_ANY;
  addr_s.sin_port = htons (PORT);
  bzero (&(addr_s.sin_zero), 8);	// nuluj zvysok struktury

  if (bind (sockfd_s, (struct sockaddr *) &addr_s, sizeof (addr_s)) == -1)
    {
      perror ("Chyba pri bind");
      exit (1);
    }

  if (listen (sockfd_s, BACKLOG) == -1)
    {
      perror ("Chyba pri listen");
      exit (1);
    }

  signal (SIGCHLD, SIG_IGN);	/* ignoruj signal SIGCHLD. SIGCHLD je generovany zakazdym, ked dcersky proces skonci alebo je zruseny */
  while (1)
    {
      printf ("server: cakam\n");
      client_len = sizeof (addr_c);
      /* accept() vytvara novy descriptor */
      sockfd_c = accept (sockfd_s, (struct sockaddr *) &addr_c, &client_len);
      if (fork () == 0)
	{
	  if ((len = recv (sockfd_c, buffer, MAXLEN - 1, 0)) == -1)
	    {
	      perror ("Chyba pri prijme spravy");
	      exit (1);
	    }
	  else
	    {
	      sleep (5);	/* teraz server akoze nieco robi */
	      buffer[len] = '\0';	//kazdy retazec musi koncit na \0
	      printf ("\nprijata sprava dlzky \"%s\" dlzky %d bytov", buffer,
		      len);
	      addr_len = sizeof (struct sockaddr);
	      if ((getpeername (sockfd_c, &addr_c, &addr_len)) == -1)
		{
		  perror ("Nepodarilo sa ziskat cislo stroja");
		}
	      printf (" z poc. %s", inet_ntoa (addr_c.sin_addr));
	    }
	  close (sockfd_c);
	  exit (0);
	}
      else
	{
	  close (sockfd_c);
	}
    }
}
